package socket

import (
	"errors"
	"sync/atomic"

	"git.loafle.net/commons/server-go"
)

type CompressionHandler interface {
	IsEnableCompression() bool
	GetCompressionLevel() int
	GetCompressionThreshold() int
}

type CompressionHandlers struct {
	EnableCompression    bool `json:"enableCompression,omitempty"`
	CompressionLevel     int  `json:"compressionLevel,omitempty"`
	CompressionThreshold int  `json:"compressionThreshold,omitempty"`

	validated atomic.Value
}

func (ch *CompressionHandlers) IsEnableCompression() bool {
	return ch.EnableCompression
}
func (ch *CompressionHandlers) GetCompressionLevel() int {
	return ch.CompressionLevel
}
func (ch *CompressionHandlers) GetCompressionThreshold() int {
	return ch.CompressionThreshold
}

func (ch *CompressionHandlers) Clone() *CompressionHandlers {
	return &CompressionHandlers{
		EnableCompression:    ch.EnableCompression,
		CompressionLevel:     ch.CompressionLevel,
		CompressionThreshold: ch.CompressionThreshold,
		validated:            ch.validated,
	}
}

func (ch *CompressionHandlers) Validate() error {
	if nil != ch.validated.Load() {
		return nil
	}
	ch.validated.Store(true)

	if !IsValidCompressionLevel(ch.CompressionLevel) {
		return errors.New("Socket: invalid compression level")
	}

	if ch.CompressionThreshold <= 0 {
		ch.CompressionThreshold = server.DefaultCompressionThreshold
	}

	return nil
}
